import React from 'react';
import Uuid from 'uuid';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import map from 'lodash/map';
import noop from 'lodash/noop';
import './bread-crumb.less';
import Icon from './../icon/icon';

export default function BreadCrumb({
    id,
    items,
    subClass
}) {
    const className = classnames(
        'tsdev-bread-crumb', subClass
    );
    const breadCrumbItems = map(items, (item, index) => {
        const innerClass = classnames('tsdev-bread-crumb-item', { disabled: item.disabled });
        const innerClassLink = classnames({ active: item.active });
        const inlineStyleIcon = {
            display: item.showIcon ? 'inline-block' : 'none'
        };
        const icon = (
            <div style={inlineStyleIcon}>
                <Icon iconType={item.iconType} />
            </div>
        );

        let onClick = noop;

        /* istanbul ignore if */
        if (!item.disabled) {
            onClick = () => {
                item.onClick({
                    item
                });
            };
        }

        return (
            <li key={`item-${index}`}
                onClick={onClick}
                className={innerClass}>
                <a className={innerClassLink}
                    href={item.link}><span className="icon">{icon}</span>{item.label}</a>
            </li>
        );
    });

    return (
        <div id={id}
            className={className}>
            <ul className="bread-crumb-list">
                {breadCrumbItems}
            </ul>
        </div>
    );
}

BreadCrumb.propTypes = {
    id: PropTypes.string,
    items: PropTypes.arrayOf(PropTypes.shape({
        label: PropTypes.string,
        disabled: PropTypes.bool,
        active: PropTypes.bool,
        link: PropTypes.string,
        linkTarget: PropTypes.string,
        onClick: PropTypes.func,
        showIcon: PropTypes.bool,
        iconType: PropTypes.string
    })),
    subClass: PropTypes.string
};

BreadCrumb.defaultProps = {
    id: `tsdevBreadCrumb-${Uuid.v4()}`,
    items: [
        {
            label: '',
            disabled: false,
            active: false,
            link: '',
            linkTarget: '_self',
            onClick: noop,
            showIcon: true,
            iconType: 'info'
        },
        {
            label: 'Item - 1',
            disabled: false,
            active: false,
            link: '',
            linkTarget: '_self',
            onClick: noop,
            showIcon: false,
            iconType: 'info'
        },
        {
            label: 'Item - 2',
            disabled: true,
            active: false,
            link: '',
            linkTarget: '_self',
            onClick: noop,
            showIcon: false,
            iconType: 'info'
        },
        {
            label: 'Item - 3',
            disabled: false,
            active: true,
            link: '',
            linkTarget: '_self',
            onClick: noop,
            showIcon: true,
            iconType: 'info'
        },
        {
            label: 'Item - 4',
            disabled: false,
            active: false,
            link: '',
            linkTarget: '_self',
            onClick: noop,
            showIcon: false,
            iconType: 'info'
        }
    ],
    subClass: ''
};

BreadCrumb.displayName = 'BreadCrumb';
