import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import Uuid from 'uuid';
import reverse from 'lodash/reverse';
import each from 'lodash/each';
import first from 'lodash/first';
import last from 'lodash/last';
import map from 'lodash/map';
import noop from 'lodash/noop';
import './datepicker.less';
import Icon from './../icon/icon';
import {
    getDaysInMonth,
    lastDayOfMonth,
    isSunday,
    isMonday,
    isTuesday,
    isWednesday,
    isThursday,
    isFriday,
    isSaturday
} from 'date-fns';

export default class Datepicker extends Component {
    constructor(props) {
        super(props);
        this.state = {
            headerLabel: '',
            startYear: props.startYear,
            months: props.months,
            days: props.days,
            isYearSelected: false,
            isMonthSelected: false,
            isDaySelected: false,
            value: this.props.value,
            year: 0,
            day: 0,
            month: 0,
            yearSelectionIndex: 0,
            selectedYearIndex: 0,
            maxYearIndex: 0,
            setInitialValues: true,
            yearGroups: [],
            daysInMonth: [],
            shiftedDaysIndex: [],
            isOpen: props.isOpen,
            closeOnSelect: props.closeOnSelect
        };
        this.onClickHeaderNavLeft = this.onClickHeaderNavLeft.bind(this);
        this.onClickHeaderNavRight = this.onClickHeaderNavRight.bind(this);
        this.onClickYear = this.onClickYear.bind(this);
        this.onClickMonth = this.onClickMonth.bind(this);
        this.onClickDay = this.onClickDay.bind(this);
    }

    static getDerivedStateFromProps(nextProps, state) {
        const yearsGroupSize = 10;
        const valueSplit = nextProps.value.split('.');
        const year = state.setInitialValues ? Number(valueSplit[2]) : /* istanbul ignore next */ state.year;
        const month = state.setInitialValues ? Number(valueSplit[1]) : /* istanbul ignore next */ state.month;
        const currentDate = new Date();
        const currentYear = currentDate.getFullYear();

        let yearGroups = [];

        let shiftedDaysIndex = [];

        let daysInMonth = [];

        // generate a two-dimensional array of years with an inner array size of 10 items
        const generateArrayOfYears = () => {
            let arrayOfYears = [];

            let yearsSection = [];

            for (let yearCount = state.startYear; yearCount < currentYear; yearCount += 1) {
                if (yearCount % yearsGroupSize === 0) {
                    if (yearCount !== state.startYear) {
                        yearGroups.push(yearsSection);
                    }
                    yearsSection = [];
                }
                yearsSection.push(yearCount);
                arrayOfYears.push(yearCount);
            }
            arrayOfYears.push(currentYear);

            return reverse(arrayOfYears);
        };

        // call the function above to generate items
        generateArrayOfYears();

        const lastYearInCalculatedGroups = yearGroups[yearGroups.length - 1][yearsGroupSize - 1];
        const yearsExtensionArray = [];

        // complete the year group array for last items
        for (let yearToAdd = lastYearInCalculatedGroups + 1; yearToAdd <= currentYear; yearToAdd += 1) {
            yearsExtensionArray.push(yearToAdd);
        }

        yearGroups = yearGroups.concat([yearsExtensionArray]);

        // set default index for selected year
        each(yearGroups, (yearGroup, outerIndex) => {
            if (yearGroup.indexOf(year) !== -1 && state.setInitialValues) {
                state.selectedYearIndex = outerIndex;
            }
        });

        if (!state.isYearSelected) {
            state.headerLabel =
                `${first(yearGroups[state.selectedYearIndex])} - ${last(yearGroups[state.selectedYearIndex])}`;
        }

        // complete the days in month
        for (let dayCount = 1;
            dayCount <= getDaysInMonth(new Date(state.setInitialValues
                ? Number(valueSplit[2])
                : /* istanbul ignore next */ state.year,
                state.setInitialValues ? Number(valueSplit[1]) - 1 : state.month - 1));
            dayCount += 1) {
            daysInMonth.push(dayCount);
        }

        const firstDayInCurrentMonth = new Date(year, month - 1, 1);
        const firstDayInLastMonth = new Date(year, month - 2, 1);

        // unshift days for right position in calendar
        /* istanbul ignore next */
        if (!isSunday(firstDayInCurrentMonth)) {
            daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)));
            shiftedDaysIndex.push(0);

            if (!isMonday(firstDayInCurrentMonth)) {
                daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 1);
                shiftedDaysIndex.push(1);

                if (!isTuesday(firstDayInCurrentMonth)) {
                    daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 2);
                    shiftedDaysIndex.push(2);

                    if (!isWednesday(firstDayInCurrentMonth)) {
                        daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 3);
                        shiftedDaysIndex.push(3);

                        if (!isThursday(firstDayInCurrentMonth)) {
                            daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 4);
                            shiftedDaysIndex.push(4);

                            if (!isFriday(firstDayInCurrentMonth)) {
                                daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 5);
                                shiftedDaysIndex.push(5);

                                /* istanbul ignore next */
                                if (!isSaturday(firstDayInCurrentMonth)) {
                                    daysInMonth.unshift(getDaysInMonth(lastDayOfMonth(firstDayInLastMonth)) - 6);
                                    shiftedDaysIndex.push(6);
                                }
                            }
                        }
                    }
                }
            }
        }

        const maxNumberOfDays = 42;
        const numberOfDays = daysInMonth.length;

        // add unselectable days to the calendar to maxNumberOfDays
        /* istanbul ignore else */
        if (numberOfDays < maxNumberOfDays) {
            let dayCounterToAdd = 1;

            for (let dayToAdd = numberOfDays; dayToAdd < maxNumberOfDays; dayToAdd += 1) {
                daysInMonth.push(dayCounterToAdd);
                shiftedDaysIndex.push(numberOfDays + (dayCounterToAdd - 1));
                dayCounterToAdd += 1;
            }
        }

        return {
            year: state.setInitialValues ? Number(valueSplit[2]) : /* istanbul ignore next */ state.year,
            month: state.setInitialValues ? Number(valueSplit[1]) : /* istanbul ignore next */ state.month,
            day: state.setInitialValues ? Number(valueSplit[0]) : /* istanbul ignore next */ state.day,
            selectedYearIndex: state.selectedYearIndex,
            maxYearIndex: yearGroups.length - 1,
            yearGroups: yearGroups,
            currentYear: currentYear,
            daysInMonth: daysInMonth,
            shiftedDaysIndex: shiftedDaysIndex,
            closeOnSelect: nextProps.closeOnSelect
        };
    }

    onClickHeaderNavLeft() {
        const {
            onClickArrowLeft
        } = this.props;
        const {
            year,
            month,
            selectedYearIndex,
            isYearSelected,
            isMonthSelected,
            isDaySelected,
            startYear
        } = this.state;

        if (selectedYearIndex > 0 && !isYearSelected && !isMonthSelected) {
            this.setState({
                selectedYearIndex: selectedYearIndex - 1,
                setInitialValues: false
            });
        }

        if (year > startYear && isYearSelected && !isMonthSelected) {
            this.setState({
                year: year - 1,
                headerLabel: year - 1,
                setInitialValues: false
            });
        }

        if (isYearSelected && isMonthSelected && month > 1) {
            this.setState({
                month: month - 1,
                setInitialValues: false,
                headerLabel: `${this.props.months[month - 2].nameLong} ${year}`
            });
        }

        onClickArrowLeft({
            id: this.props.id,
            year: this.state.year - 1,
            month: this.state.month - 1,
            day: this.state.day,
            value: this.state.value,
            isYearSelected,
            isMonthSelected,
            isDaySelected
        });
    }

    onClickHeaderNavRight() {
        const {
            onClickArrowRight
        } = this.props;
        const {
            selectedYearIndex,
            isYearSelected,
            isMonthSelected,
            maxYearIndex,
            currentYear,
            year,
            month,
            isDaySelected
        } = this.state;

        if (selectedYearIndex < maxYearIndex && !isYearSelected) {
            this.setState({
                selectedYearIndex: selectedYearIndex + 1,
                setInitialValues: false
            });
        }

        if (year < currentYear && isYearSelected && !isMonthSelected) {
            this.setState({
                year: year + 1,
                headerLabel: year + 1
            });
        }

        if (isYearSelected && isMonthSelected && month < 12) {
            this.setState({
                month: month + 1,
                setInitialValues: false,
                headerLabel: `${this.props.months[month].nameLong} ${year}`
            });
        }

        onClickArrowRight({
            id: this.props.id,
            year: this.state.year,
            month: this.state.month + 1,
            day: this.state.day,
            value: this.state.value,
            isYearSelected,
            isMonthSelected,
            isDaySelected
        });
    }

    onClickYear(event) {
        const year = Number(event.target.getAttribute('data-attribute-year'));

        this.setState({
            year: year,
            isYearSelected: true,
            headerLabel: year,
            setInitialValues: false
        });

        this.props.onClickYear({
            id: this.props.id,
            year: year,
            month: this.state.month,
            day: this.state.day,
            value: this.state.value,
            isYearSelected: true,
            isMonthSelected: false,
            isDaySelected: false
        });
    }

    onClickMonth(event) {
        const {
            year
        } = this.state;
        const month = Number(event.target.getAttribute('data-attribute-month'));
        const monthName = event.target.getAttribute('data-attribute-month-name');

        this.setState({
            month: month,
            isYearSelected: true,
            isMonthSelected: true,
            headerLabel: `${monthName} ${year}`,
            setInitialValues: false
        });

        this.props.onClickMonth({
            id: this.props.id,
            year: year,
            month: month,
            day: this.state.day,
            value: this.state.value,
            isYearSelected: true,
            isMonthSelected: true,
            isDaySelected: false
        });
    }

    onClickDay(event) {
        const {
            year,
            month,
            closeOnSelect
        } = this.state;
        const selectedDay = Number(event.target.getAttribute('data-attribute-day'));
        const zeroPrefixDay = selectedDay < 10 ? /* istanbul ignore next */ '0' : '';
        const zeroPrefixMonth = month < 10 ? '0' : /* istanbul ignore next */ '';
        const calculatedValue = `${zeroPrefixDay}${selectedDay}.${zeroPrefixMonth}${month}.${year}`;

        this.setState({
            day: selectedDay,
            isYearSelected: false,
            isMonthSelected: false,
            isDaySelected: true && closeOnSelect,
            setInitialValues: true && closeOnSelect,
            value: calculatedValue,
            isOpen: !closeOnSelect
        });

        this.props.onClickDay({
            id: this.props.id,
            year: year,
            month: month,
            day: selectedDay,
            value: calculatedValue,
            isOpen: !closeOnSelect,
            isYearSelected: true,
            isMonthSelected: true,
            isDaySelected: true
        });
    }


    render() {
        const {
            id,
            showMonthLongNames,
            subClass
        } = this.props;
        const {
            headerLabel,
            yearGroups,
            selectedYearIndex,
            maxYearIndex,
            isYearSelected,
            year,
            month,
            startYear,
            currentYear,
            isMonthSelected,
            isDaySelected,
            isOpen
        } = this.state;
        const inlineStyle = {
            display: isOpen ? 'block' : /* istanbul ignore next */ 'none'
        };
        const inlineStyleYearGroups = {
            display: !isYearSelected && !isMonthSelected && !isDaySelected ? 'flex' : 'none'
        };
        const inlineStyleMonthWrapper = {
            display: isYearSelected && !isMonthSelected && !isDaySelected ? 'flex' : 'none'
        };
        const inlineStyleDaysWrapper = {
            display: isYearSelected && isMonthSelected && !isDaySelected ? 'flex' : 'none'
        };
        const className = classnames('tsdev-datepicker', subClass);
        const classNameArrowLeft = classnames(
            'header-nav-icon',
            'arrow-left',
            {
                disabled: selectedYearIndex === 0 && /* istanbul ignore next */ !isYearSelected
                    || year === startYear
                    || (isMonthSelected && month === 1)
            }
        );
        const classNameArrowRight = classnames(
            'header-nav-icon',
            'arrow-right',
            {
                disabled: selectedYearIndex === maxYearIndex && /* istanbul ignore next */ !isYearSelected
                    || year === currentYear && /* istanbul ignore next */ isYearSelected
                    || (isMonthSelected && month === 12)
            }
        );
        const yearGroup = map(yearGroups[selectedYearIndex], (yearItem, index) => {
            const itemClassName = classnames('year-group-item', { selected: year === yearItem });

            return (
                <div className={itemClassName}
                    data-attribute-year={yearItem}
                    onClick={this.onClickYear}
                    key={`year-group-item-${index}`}>
                    {yearItem}
                </div>
            );
        });
        const yearGroupContent = (
            <div className="years-wrapper"
                style={inlineStyleYearGroups}>
                <div className="year-group-item"
                    onClick={this.onClickYear}
                    data-attribute-year={first(yearGroups[selectedYearIndex]) - 1}>
                    {first(yearGroups[selectedYearIndex]) - 1}
                </div>
                {yearGroup}
                <div className="year-group-item"
                    onClick={this.onClickYear}
                    data-attribute-year={last(yearGroups[selectedYearIndex]) + 1}>
                    {last(yearGroups[selectedYearIndex]) + 1}
                </div>
            </div>
        );
        const renderMonthContent = () => {
            const {
                months
            } = this.state;

            return map(months, (monthItem, index) => {
                const monthClassName = classnames(
                    'month-item',
                    { selected: month === index + 1 }
                );

                return (
                    <div key={`month-item-${index}`}
                        data-attribute-month={index + 1}
                        data-attribute-month-name={monthItem.nameLong}
                        onClick={this.onClickMonth}
                        className={monthClassName}>
                        {showMonthLongNames ? /* istanbul ignore next */ monthItem.nameLong : monthItem.nameShort}
                    </div>
                );
            });
        };
        const renderDayNames = () => {
            const {
                days
            } = this.state;

            return map(days, (dayItem, index) => {
                return (
                    <div key={`day-name-item-${index}`}
                        className="day-name-item">
                        {dayItem.nameShort}
                    </div>
                );
            });
        };
        const renderDays = () => {
            const {
                day,
                daysInMonth,
                shiftedDaysIndex
            } = this.state;

            return map(daysInMonth, (dayInMonthItem, index) => {
                const dayClassName = classnames(
                    'day-item',
                    { selected: dayInMonthItem === day },
                    { 'not-available': shiftedDaysIndex.indexOf(index) !== -1 },
                    {
                        hidden: !this.props.fillDaysSection
                            && /* istanbul ignore next */ shiftedDaysIndex.indexOf(index) !== -1
                    });

                return (
                    <div key={`day-item-${index}`}
                        className={dayClassName}
                        data-attribute-day={dayInMonthItem}
                        onClick={this.onClickDay}>
                        {dayInMonthItem}
                    </div>
                );
            });
        };

        return (
            <div id={id}
                className={className}
                style={inlineStyle}>
                <div className="header-section">
                    <div className="header-section-top" />
                    <div className="header-section-content">
                        <span className={classNameArrowLeft}
                            id="headerNavIconLeft"
                            onClick={this.onClickHeaderNavLeft}>
                            <Icon iconType="arrow-left"
                                size={18} />
                        </span>
                        <span className="header-label">
                            {headerLabel}
                        </span>
                        <span className={classNameArrowRight}
                            id="headerNavIconRight"
                            onClick={this.onClickHeaderNavRight}>
                            <Icon iconType="arrow-right"
                                size={18} />
                        </span>
                    </div>
                </div>
                <div className="content-section">
                    <div className="content-section-inner">
                        {yearGroupContent}
                        <div className="months-wrapper"
                            style={inlineStyleMonthWrapper}>
                            {renderMonthContent()}
                        </div>
                        <div className="days-wrapper"
                            style={inlineStyleDaysWrapper}>
                            <div className="day-names">
                                {renderDayNames()}
                            </div>
                            <div className="days">
                                {renderDays()}
                            </div>
                        </div>
                    </div>
                </div>
                <div className="footer-section" />
            </div>
        );
    }
}

Datepicker.propTypes = {
    closeOnSelect: PropTypes.bool,
    days: PropTypes.array,
    fillDaysSection: PropTypes.bool,
    id: PropTypes.string,
    isOpen: PropTypes.bool,
    months: PropTypes.array,
    onClickArrowLeft: PropTypes.func,
    onClickArrowRight: PropTypes.func,
    onClickDay: PropTypes.func,
    onClickMonth: PropTypes.func,
    onClickYear: PropTypes.func,
    showMonthLongNames: PropTypes.bool,
    startYear: PropTypes.number,
    subClass: PropTypes.string,
    value: PropTypes.string
};

Datepicker.defaultProps = {
    closeOnSelect: false,
    days: [
        {
            nameLong: 'Sunday',
            nameShort: 'Su'
        },
        {
            nameLong: 'Monday',
            nameShort: 'Mo'
        },
        {
            nameLong: 'Tuesday',
            nameShort: 'Tu'
        },
        {
            nameLong: 'Wednesday',
            nameShort: 'We'
        },
        {
            nameLong: 'Thursday',
            nameShort: 'Th'
        },
        {
            nameLong: 'Friday',
            nameShort: 'Fr'
        },
        {
            nameLong: 'Saturday',
            nameShort: 'Sa'
        }
    ],
    fillDaysSection: true,
    id: `tsdevDatepicker-${Uuid.v4()}`,
    isOpen: true,
    months: [
        {
            nameLong: 'January',
            nameShort: 'Jan'
        },
        {
            nameLong: 'February',
            nameShort: 'Feb'
        },
        {
            nameLong: 'March',
            nameShort: 'Mar'
        },
        {
            nameLong: 'April',
            nameShort: 'Apr'
        },
        {
            nameLong: 'May',
            nameShort: 'May'
        },
        {
            nameLong: 'June',
            nameShort: 'Jun'
        },
        {
            nameLong: 'July',
            nameShort: 'Jul'
        },
        {
            nameLong: 'August',
            nameShort: 'Aug'
        },
        {
            nameLong: 'September',
            nameShort: 'Sep'
        },
        {
            nameLong: 'October',
            nameShort: 'Oct'
        },
        {
            nameLong: 'November',
            nameShort: 'Nov'
        },
        {
            nameLong: 'December',
            nameShort: 'Dec'
        }
    ],
    onClickArrowLeft: noop,
    onClickArrowRight: noop,
    onClickMonth: noop,
    onClickDay: noop,
    onClickYear: noop,
    showMonthLongNames: false,
    startYear: 1800,
    subClass: '',
    value: '18.02.1989'
};

Datepicker.displayName = 'Datepicker';
