import React, { useState, useEffect } from 'react';
import Uuid from 'uuid';
import classnames from 'classnames';
import noop from 'lodash/noop';
import PropTypes from 'prop-types';
import './panel.less';
import Icon from './../icon/icon';

export default function Panel({
    children,
    id,
    subClass,
    title,
    width,
    expanded,
    contentHeight,
    onToggle
}) {
    const [isExpandedState, setIsExpandedState] = useState(expanded);
    const inlineStyle = {
        width: width
    };
    const inlineStyleContent = {
        height: contentHeight
    };
    const iconType = isExpandedState ? 'arrow-down' : 'arrow-right';
    const togglePanelContent = () => {
        setIsExpandedState(!isExpandedState);
        onToggle({
            id,
            expanded: !isExpandedState
        });
    };
    const className = classnames(
        'tsdev-panel', { expanded: isExpandedState }, subClass
    );
    const classNameHeader = classnames('panel-header', { expanded: isExpandedState });
    const classNameContent = classnames('panel-content', { expanded: isExpandedState });

    useEffect(() => {
        setIsExpandedState(expanded);
    }, [expanded]);

    return (
        <div id={id}
            className={className}
            style={inlineStyle} >
            <div className={classNameHeader}
                onClick={togglePanelContent}>
                <div className="panel-header-toggle-icon">
                    <Icon size={20}
                        iconType={iconType} />
                </div>
                <div className="panel-header-title">
                    {title}
                </div>
            </div>
            <div className={classNameContent}
                style={inlineStyleContent}>
                {children}
            </div>
        </div>
    );
}

Panel.propTypes = {
    children: PropTypes.node,
    contentHeight: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    expanded: PropTypes.bool,
    id: PropTypes.string,
    onToggle: PropTypes.func,
    subClass: PropTypes.string,
    title: PropTypes.string,
    width: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ])
};

Panel.defaultProps = {
    children: (<div>children</div>),
    contentHeight: 250,
    expanded: false,
    id: `tsdevPanel-${Uuid.v4()}`,
    onToggle: noop,
    subClass: '',
    title: 'Panel',
    width: 500
};

Panel.displayName = 'Panel';
